@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,16 +15,45 @@
  *
 */
 
-#ifndef _USE_MATH_DEFINES
-#define _USE_MATH_DEFINES
-#endif
 #include <gtest/gtest.h>
 #include <cmath>
 
-#include "ignition/math/Helpers.hh"
-#include "ignition/math/Vector2.hh"
+#include "gz/math/Helpers.hh"
+#include "gz/math/Vector2.hh"
 
-using namespace ignition;
+using namespace gz;
+
+/////////////////////////////////////////////////
+TEST(Vector2Test, Construction)
+{
+  math::Vector2d vec(1, 0);
+
+  // Copy constructor
+  math::Vector2d vec2(vec);
+  EXPECT_EQ(vec2, vec);
+
+  // Copy operator
+  math::Vector2d vec3;
+  vec3 = vec;
+  EXPECT_EQ(vec3, vec);
+
+  // Move constructor
+  math::Vector2d vec4(std::move(vec));
+  EXPECT_EQ(vec4, vec2);
+  vec = vec4;
+  EXPECT_EQ(vec, vec2);
+
+  // Move operator
+  math::Vector2d vec5;
+  vec5 = std::move(vec2);
+  EXPECT_EQ(vec5, vec3);
+  vec2 = vec5;
+  EXPECT_EQ(vec2, vec3);
+
+  // Inequality
+  math::Vector2d vec6;
+  EXPECT_NE(vec6, vec3);
+}
 
 /////////////////////////////////////////////////
 TEST(Vector2Test, Vector2)
@@ -47,10 +76,22 @@ TEST(Vector2Test, Vector2)
   v.Normalize();
   EXPECT_TRUE(v == math::Vector2d(0.447214, 0.894427));
 
+  // ::Rounded
+  v.Set(3.55, 8.49);
+  EXPECT_TRUE(v.Rounded() == math::Vector2d(4, 8));
+
+  // ::Round
+  v.Round();
+  EXPECT_TRUE(v == math::Vector2d(4, 8));
+
   // ::Set
   v.Set(4, 5);
   EXPECT_TRUE(v == math::Vector2d(4, 5));
 
+  // operator GetAbs
+  v.Set(-1, -2);
+  EXPECT_TRUE(v.Abs() == math::Vector2d(1, 2));
+
   // ::operator=
   v = math::Vector2d(6, 7);
   EXPECT_TRUE(v == math::Vector2d(6, 7));
@@ -116,24 +157,91 @@ TEST(Vector2Test, Vector2)
   v *= math::Vector2d(2, 4);
   EXPECT_TRUE(v == math::Vector2d(20, 24));
 
-
   // ::IsFinite
   EXPECT_TRUE(v.IsFinite());
 
   // ::operator[]
-  v.Set(6, 7);
+  v[0] = 6;
+  v[1] = 7;
   EXPECT_DOUBLE_EQ(6, v[0]);
   EXPECT_DOUBLE_EQ(7, v[1]);
 }
 
 /////////////////////////////////////////////////
-TEST(Vector2Test, IndexException)
+TEST(Vector2Test, TestSum)
+{
+  math::Vector2 vec1(0, 0);
+  math::Vector2 vec2(1.0, 2.5);
+  math::Vector2 vec3(-2, -4);
+
+  int sum1 = vec1.Sum();
+  float sum2 = vec2.Sum();
+  int sum3 = vec3.Sum();
+
+  EXPECT_EQ(sum1, 0);
+  EXPECT_FLOAT_EQ(sum2, 3.5);
+  EXPECT_EQ(sum3, -6);
+}
+
+/////////////////////////////////////////////////
+TEST(Vector2Test, TestNormalized)
+{
+  math::Vector2d vec1(0, 0);
+  math::Vector2d vec2(1, 2);
+
+  math::Vector2d vec3 = vec1.Normalized();
+
+  // O zero vector should be equal to the normalized vector
+  EXPECT_EQ(vec1, vec3);
+  EXPECT_NE(vec2, vec3);
+
+  vec3 = vec2.Normalized();
+  EXPECT_EQ(vec3, math::Vector2d(0.447213, 0.894427));
+}
+
+/////////////////////////////////////////////////
+TEST(Vector2Test, Max)
+{
+  math::Vector2d vec1(0.1, 0.2);
+  math::Vector2d vec2(0.3, 0.5);
+  math::Vector2d vec3(0.4, 0.2);
+
+  EXPECT_DOUBLE_EQ(vec1.Max(), 0.2);
+  EXPECT_DOUBLE_EQ(vec3.Max(), 0.4);
+
+  vec1.Max(vec2);
+  EXPECT_EQ(vec1, math::Vector2d(0.3, 0.5));
+
+  vec1.Max(vec3);
+  EXPECT_EQ(vec1, math::Vector2d(0.4, 0.5));
+}
+
+/////////////////////////////////////////////////
+TEST(Vector2Test, Min)
+{
+  math::Vector2d vec1(0.3, 0.5);
+  math::Vector2d vec2(0.1, 0.2);
+  math::Vector2d vec3(0.05, 0.1);
+
+  EXPECT_DOUBLE_EQ(vec1.Min(), 0.3);
+  EXPECT_DOUBLE_EQ(vec3.Min(), 0.05);
+
+  vec1.Min(vec2);
+  EXPECT_EQ(vec1, math::Vector2d(0.1, 0.2));
+
+  vec1.Min(vec3);
+  EXPECT_EQ(vec1, math::Vector2d(0.05, 0.1));
+}
+
+/////////////////////////////////////////////////
+TEST(Vector2Test, NoException)
 {
   math::Vector2d v(1, 2);
   EXPECT_NO_THROW(math::equal(v[0], 1.0));
   EXPECT_NO_THROW(math::equal(v[1], 2.0));
 
-  EXPECT_THROW(math::equal(v[2], 1.0), math::IndexException);
+  EXPECT_NO_THROW(math::equal(v[2], 1.0));
+  EXPECT_DOUBLE_EQ(v[2], 2.0);
 }
 
 /////////////////////////////////////////////////
@@ -158,6 +266,33 @@ TEST(Vector2Test, Dot)
   EXPECT_DOUBLE_EQ(v.Dot(math::Vector2d(0, 1)), 2.0);
 }
 
+//////////////////////////////////////////////
+TEST(Vector2Test, Correct)
+{
+  math::Vector2d vec1(0, NAN);
+  math::Vector2d vec2(INFINITY, -1);
+  math::Vector2d vec3(10, -2);
+
+  vec1.Correct();
+  vec2.Correct();
+  vec3.Correct();
+
+  EXPECT_EQ(vec1, math::Vector2d(0, 0));
+  EXPECT_EQ(vec2, math::Vector2d(0, -1));
+  EXPECT_EQ(vec3, math::Vector2d(10, -2));
+}
+
+/////////////////////////////////////////////////
+TEST(Vector2Test, AbsDot)
+{
+  math::Vector2d v(1, -2);
+
+  EXPECT_DOUBLE_EQ(v.AbsDot(math::Vector2d(3, 4)), 11.0);
+  EXPECT_DOUBLE_EQ(v.AbsDot(math::Vector2d(0, 0)), 0.0);
+  EXPECT_DOUBLE_EQ(v.AbsDot(math::Vector2d(1, 0)), 1.0);
+  EXPECT_DOUBLE_EQ(v.AbsDot(math::Vector2d(0, 1)), 2.0);
+}
+
 /////////////////////////////////////////////////
 TEST(Vector2Test, OperatorStreamOut)
 {
@@ -168,7 +303,7 @@ TEST(Vector2Test, OperatorStreamOut)
 }
 
 /////////////////////////////////////////////////
-TEST(Vector2dTest, Add)
+TEST(Vector2Test, Add)
 {
   math::Vector2d vec1(0.1, 0.2);
   math::Vector2d vec2(1.1, 2.2);
@@ -209,7 +344,7 @@ TEST(Vector2dTest, Add)
 }
 
 /////////////////////////////////////////////////
-TEST(Vector2dTest, Sub)
+TEST(Vector2Test, Sub)
 {
   math::Vector2d vec1(0.1, 0.2);
   math::Vector2d vec2(1.1, 2.2);
@@ -287,19 +422,46 @@ TEST(Vector2Test, Multiply)
 }
 
 /////////////////////////////////////////////////
-TEST(Vector2dTest, Length)
+TEST(Vector2Test, Length)
 {
   // Zero vector
   EXPECT_DOUBLE_EQ(math::Vector2d::Zero.Length(), 0.0);
   EXPECT_DOUBLE_EQ(math::Vector2d::Zero.SquaredLength(), 0.0);
 
   // One vector
-  EXPECT_NEAR(math::Vector2d::One.Length(), M_SQRT2, 1e-10);
+  EXPECT_NEAR(math::Vector2d::One.Length(), IGN_SQRT2, 1e-10);
   EXPECT_DOUBLE_EQ(math::Vector2d::One.SquaredLength(), 2.0);
 
   // Arbitrary vector
   math::Vector2d v(0.1, -4.2);
   EXPECT_NEAR(v.Length(), 4.20119030752, 1e-10);
   EXPECT_DOUBLE_EQ(v.SquaredLength(), 17.65);
+
+  // Integer vector
+  math::Vector2i vi(3, 4);
+  EXPECT_EQ(vi.Length(), 5);
+  EXPECT_EQ(vi.SquaredLength(), 25);
+}
+
+/////////////////////////////////////////////////
+TEST(Vector2Test, NaN)
+{
+  auto nanVec = math::Vector2d::NaN;
+  EXPECT_FALSE(nanVec.IsFinite());
+  EXPECT_TRUE(math::isnan(nanVec.X()));
+  EXPECT_TRUE(math::isnan(nanVec.Y()));
+
+  nanVec.Correct();
+  EXPECT_EQ(math::Vector2d::Zero, nanVec);
+  EXPECT_TRUE(nanVec.IsFinite());
+
+  auto nanVecF = math::Vector2f::NaN;
+  EXPECT_FALSE(nanVecF.IsFinite());
+  EXPECT_TRUE(math::isnan(nanVecF.X()));
+  EXPECT_TRUE(math::isnan(nanVecF.Y()));
+
+  nanVecF.Correct();
+  EXPECT_EQ(math::Vector2f::Zero, nanVecF);
+  EXPECT_TRUE(nanVecF.IsFinite());
 }
 
